#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


from pathlib import Path
import wx
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
from nardascripting.base.measthread import *
import nardascripting.base.toolbox as tb
import time
from lxml import etree as ET
import csv


class ConvertTxTab(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this line unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Script settings.
        # -------------------------------------------------------------------------------
        # Please adapt the following lines of code according to your script.
        self._tab_name = "nslconverter"
        self._scr_title = "Convert IDA to SignalShark tx table V1.1"
        self._scr_description = "Converts IDA CSV based Tx table to XML based SignalShark Tx table V1.0"
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_example01.png")
        self._list_prio = 3
        self._nsl_executed_behavior = NSL_Executed_Behaviors.MINIMIZE_NSL

        # Add class variables if needed
        # -------------------------------------------------------------------------------
        # self.my_variable = None

    def _run_script(self, args):
        """
        Script main function
        This method is called when a user clicks on the corresponding script button.
        """

        # Show file browser for file selection:
        dlg = wx.FileDialog(self.frm_dlghelper, "Open IDA Tx table", "", "",
                            "CSV files (*.csv)|*.csv",
                            wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
        self.ShowDlgModalTop(dlg)
        file2convert = dlg.GetPath()
        dlg.Destroy()
        if not file2convert:
            self.MessageBoxModal("No valid file selected.", "Info",
                                 wx.OK | wx.ICON_INFORMATION)
            return
        file_name = file2convert

        # Determine destination folder:
        csv_file_path = Path(file_name)
        result_folder = csv_file_path.parent
        SignalShark_txtab_path = Path("D:\\Narda_SignalShark\\Configuration\\TransmitterTables")
        if SignalShark_txtab_path.exists():
            if self.MessageBoxModal(
                    "Should the result be saved directly "
                    "into the transmitter table folder of SignalShark ({})?".format(SignalShark_txtab_path),
                    "Save to TxTable folder?", wx.YES_NO | wx.ICON_QUESTION) == wx.YES:
                result_folder = SignalShark_txtab_path

        # Create an additional measurement thread to keep the GUI alive.
        # With "args=[param1, parm2, paramn]" you can pass some optional parameters to the measurement thread.
        mthread = MeasDlgThread(main_gui=self.main_gui, callback=ConvertTxTab._run_measurement,
                                args=[file_name, result_folder])

        # Start the measurement thread and show a visualization (progress) dialog.
        dlg_result = mthread.start_measurement()
        if dlg_result == wx.ID_OK:
            self.main_gui.MessageBoxModal("Conversion done.", "Info")
        else:
            self.main_gui.MessageBoxModal("Conversion canceled.", "Info")

    @staticmethod
    def _run_measurement(stopevent, update_progress, wait_for_stopevent, file_name: str, result_folder: Path):
        """ Main method of the measurement thread that handles a time consuming measurement.

        :param stopevent: Thread flag that indicates whether the procedure should be finished/cancelled
        :param update_progress: Delegate method to update a progress bar, message string and icon
        :param wait_for_stopevent: Delegate method to wait until stopevent is raised with possibility to show a message.
        :param file_name: Custom parameter to handover the file(name) to convert.
        :param result_folder: Custom parameter to handover the target folder.
        :return:
        """
        # Prepare progress value calculation. Progress is a value from 0.0 to 1.0
        # Open csv file:
        # -------------------------------------------------------------------------------
        data_file = None
        try:
            # Generate xml filename:
            # -------------------------------------------------------------------------------
            csv_file_path = Path(file_name)
            txtable_name = csv_file_path.name.replace(".csv", "")
            xml_file_name = result_folder.joinpath(txtable_name + ".xml")

            # Prepare progress value calculation. Progress is a value from 0.0 to 1.0
            # Open csv file:
            # -------------------------------------------------------------------------------
            data_file = open(file_name, "r")
            csv_reader = csv.reader(data_file, delimiter=";")

            # Get number of lines for progress bar
            lines = sum(1 for _ in csv_reader)
            data_file.seek(0)
            progress = 0.0
            progress_step = 1.0 / lines

            # Show Message "Conversion in progress"
            update_progress(msg="Conversion in progress", progress=0.0, btn_style=wx.CANCEL)

            # Create xml root node
            xml_doc = ET.Element('Narda_3300_TransmitterTable')

            count = 1
            label = 1
            check_cnt = 0
            longname = ""
            # Create xml additional nodes with corresponding attributes
            for _ in csv_reader:

                # Periodically check whether the user has clicked the cancel button,
                # If yes, exit thread after allowing the stop event to synchronize.
                stopevent.wait(0.05)
                if stopevent.is_set():
                    return

                if count == 1:
                    longname = _[1]

                elif count == 2:
                    shortname = _[1]
                    sub_node = ET.SubElement(xml_doc, 'Info')
                    ET.SubElement(sub_node, 'ShortName', type='string', size='1').text = str(shortname)
                    ET.SubElement(sub_node, 'Comment', type='string', size='1').text = str(longname)

                elif count == 3:
                    sub_node = ET.SubElement(xml_doc, 'Data')
                    ET.SubElement(sub_node, 'SortCriterion', type='ENUM_PARAM_TRANSMITTER_TABLE_SORT_CRITERIA', size='1'
                                  ).text = 'TRANSMITTER_FCENT'
                    ET.SubElement(sub_node, 'SortOrder', type='ENUM_PARAM_TRANSMITTER_TABLE_SORT_ORDER', size='1'
                                  ).text = 'ASC'
                    ET.SubElement(sub_node, 'NoOfTransmitters', type='ulong', size='1'
                                  ).text = str(lines - 3)

                else:
                    sub_node = ET.SubElement(xml_doc, 'Tx{:0>4}'.format(label))
                    ET.SubElement(sub_node, 'Name', type='string', size='1').text = _[0]
                    ET.SubElement(sub_node, 'Fcent', type='double', size='1').text = _[1]
                    ET.SubElement(sub_node, 'CBW', type='double', size='1').text = _[2]
                    ET.SubElement(sub_node, 'ExpectedSignalType', type='string', size='1')
                    ET.SubElement(sub_node, 'Comment', type='string', size='1')
                    ET.SubElement(sub_node, 'AnalogAudioDemodType', type='ENUM_PARAM_ANALOG_AUDIO_DEMOD_TYPES', size='1'
                                  ).text = 'UNKNWON'
                    label += 1
                count += 1
                progress += progress_step
                update_progress(progress=progress)

            # Retaining xml tree structure and defining encoding
            tree_output = ET.tostring(xml_doc, pretty_print=True, xml_declaration=True, encoding="UTF-16")

            # Writing data in xml file
            with open(str(xml_file_name), 'wb') as f:
                f.write(tree_output)

            # Show message:
            update_progress(msg="Conversion done", progress=1.0, btn_style=wx.OK)
            time.sleep(1)
        except Exception as e:
            # Do some error handling
            return wait_for_stopevent(str(e))
        finally:
            # Close file and connection.
            if data_file:
                data_file.close()
