#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


from pathlib import Path
import wx
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
from nardascripting.base.measthread import *
import nardascripting.base.toolbox as tb
import time
import h5py
import csv
import numpy as np


class ConvertSgramToCSV(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this line unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Script settings.
        # -------------------------------------------------------------------------------
        # Please adapt the following lines of code according to your script.
        self._tab_name = "nslconverter"
        self._scr_title = "Convert SignalShark Spectrogram Data to CSV V1.1"
        self._scr_description = "Converts SignalShark spectrogram data from HDF5-file to CSV-file(s). " \
                                "Several Tasks will be divided into separate CSV files."
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_example01.png")
        self._list_prio = 2
        self._nsl_executed_behavior = NSL_Executed_Behaviors.MINIMIZE_NSL

        # Add class variables if needed
        # -------------------------------------------------------------------------------
        # self.my_variable = None

    def _run_script(self, args):
        """
        Script main function
        This method is called when a user clicks on the corresponding script button.
        """

        # Show file browser for file selection:
        dlg = wx.FileDialog(self.frm_dlghelper, "Open SignalShark HDF5 file", "", "",
                            "HDF5 files (*.h5)|*.h5",
                            wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)

        self.ShowDlgModalTop(dlg)
        file2convert = dlg.GetPath()
        dlg.Destroy()
        if not file2convert:
            self.MessageBoxModal("No valid file selected.", "Info",
                                 wx.OK | wx.ICON_INFORMATION)
            return

        filepath = Path(file2convert)

        # Create an additional measurement thread to keep the GUI alive.
        # With "args=[param1, parm2, paramn]" you can pass some optional parameters to the measurement thread.
        mthread = MeasDlgThread(main_gui=self.main_gui, callback=ConvertSgramToCSV._run_measurement,
                                args=[filepath])

        # Start the measurement thread and show a visualization (progress) dialog.
        dlg_result = mthread.start_measurement()

        if dlg_result == wx.ID_OK:
            self.main_gui.MessageBoxModal("Conversion done.", "Info")
        else:
            self.main_gui.MessageBoxModal("Conversion canceled.", "Info")

    @staticmethod
    def _run_measurement(stopevent, update_progress, wait_for_stopevent, filepath: Path):
        """ Main method of the measurement thread that handles a time consuming measurement.

        :param stopevent: Thread flag that indicates whether the procedure should be finished/cancelled
        :param update_progress: Delegate method to update a progress bar, message string and icon
        :param wait_for_stopevent: Delegate method to wait until stopevent is raised with possibility to show a message.
        :param filepath: Custom parameter
        :return:
        """
        # Initialize return value (if any)
        # succeeded = False
        try:
            # Prepare progress value calculation. Progress is a value from 0.0 to 1.0
            steps = 0.0
            with h5py.File(str(filepath), 'r') as f:
                # Loop through all Tasks
                for task_id in f:
                    task = f.get(task_id)
                    task_type = str(task.attrs["Task type"][0]).replace("\'", "")
                    # Check if file contains a (RT) Spectrum Task:
                    if task_type == "bSpectrum" or task_type == "bRT Spectrum":

                        # Check if (RT) Spectrum Task contains a Spectrogram
                        Spectrogram = task.get("Spectrogram")
                        if Spectrogram:
                            steps += len(Spectrogram)

            update_cnt = 0
            progress = 0.0
            progress_step = 1.0 / steps

            # Show Message "Conversion in progress"
            update_progress(msg="Conversion in progress", progress=0.0, btn_style=wx.CANCEL)

            # Open file to convert:
            with h5py.File(str(filepath), 'r') as f:
                # Loop through all Tasks
                for task_id in f:
                    task = f.get(task_id)
                    task_type = str(task.attrs["Task type"][0]).replace("\'", "")

                    # Check if file contains a (RT) Spectrum Task:
                    if task_type == "bSpectrum" or task_type == "bRT Spectrum":

                        # Check if (RT) Spectrum Task contains a Spectrogram
                        Spectrogram = task.get("Spectrogram")
                        if Spectrogram:
                            # Create name for new csv file:
                            filename = Path(filepath.parent).joinpath(
                                filepath.stem + "-" + task_id + "-Spectrogram.csv")

                            # Write data to csv file
                            # -----------------------------------------------------------------------------------------
                            with open(filename, 'w', newline='') as f2:
                                writer = csv.writer(f2, delimiter=',')
                                # Write csv header information for measurement settings
                                writer.writerow(["Description", "Value"])
                                writer.writerow(["Binwidth / Hz", Spectrogram.attrs["Binwidth / Hz"][0]])
                                writer.writerow(
                                    ["Frequency of first bin / Hz",
                                     Spectrogram.attrs["Frequency of first bin / Hz"][0]])
                                writer.writerow(["Number of bins", Spectrogram.attrs["Number of bins"][0]])
                                writer.writerow(["Number of frames", Spectrogram.attrs["Number of frames"][0]])
                                writer.writerow(["Level unit", Spectrogram.attrs["Level unit"][0]])
                                writer.writerow(["Measurement time / s", Spectrogram.attrs["Measurement time / s"][0]])
                                writer.writerow(["RBW / Hz", Spectrogram.attrs["RBW / Hz"][0]])
                                writer.writerow(["Reference level", Spectrogram.attrs["Reference level"][0]])
                                writer.writerow(["Attenuator / dB", Spectrogram.attrs["Attenuator / dB"][0]])
                                writer.writerow(["Geolocation latitude / degree",
                                                 Spectrogram.attrs["Geolocation latitude / degree"][0]])
                                writer.writerow(["Geolocation longitude / degree",
                                                 Spectrogram.attrs["Geolocation longitude / degree"][0]])

                                # Add 2 lines space here
                                spacer = 0
                                while spacer < 2:
                                    writer.writerow("")
                                    spacer += 1

                                # Write csv header row for spectrogram table
                                writer.writerow(list(Spectrogram.dtype.names))

                                # Write spectrogram data:
                                for row_data in Spectrogram:
                                    row_list = []
                                    count = len(row_data)
                                    for x in range(count):
                                        row_list.append(np.array2string(row_data[x],
                                                                        max_line_width=1000000000, separator=','))
                                    writer.writerow(row_list)

                                    # calculate actual progress and update dialog all 100 steps
                                    progress += progress_step
                                    update_cnt += 1
                                    if update_cnt == 100:
                                        update_cnt = 0
                                        update_progress(progress=progress)

                                        # Allow the thread to synchronize the stop event.
                                        stopevent.wait(0.05)
                                        # Periodically check, whether the user has clicked the cancel button,
                                        # and exit thread if so.
                                        if stopevent.is_set():
                                            return

            # Show message:
            update_progress(msg="Conversion done", progress=1.0, btn_style=wx.OK)
            time.sleep(1)
        except Exception as e:
            # Do some error handling
            return wait_for_stopevent(str(e))
